/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2013 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
***************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file startup.c
*
* @author Freescale
*
* @version 0.0.1
*
* @date Jun. 25, 2013
*
* @brief Implement generic Kinetis startup code.
*
*******************************************************************************/

#include "common.h"

/********************************************************************/
void
common_startup(void)
{
    extern char __bss_end__[];
    extern char __bss_start__[];
    extern char _sdata[];
    extern char _edata[];
    extern char ___ROM_AT[];

    /* Declare a counter we'll use in all of the copy loops */
    uint32 n;

    /* Get the addresses for the .data section (initialized data section) */
    uint8* data_ram = (uint8*)_sdata;
    uint8* data_ram_end = (uint8*)_edata;
    uint8* data_rom = (uint8*)___ROM_AT;

    /* Copy initialized data from ROM to RAM */
    n = data_ram_end - data_ram;
    while (n--)
      *data_ram++ = *data_rom++;


    /* Get the addresses for the .bss section (zero-initialized data) */
    uint8* bss_start = (uint8*)__bss_start__;
    uint8* bss_end = (uint8*)__bss_end__;

    /* Clear the zero-initialized data section */
    n = bss_end - bss_start;
    while(n--)
      *bss_start++ = 0;
}
/********************************************************************/

